package ru.yandex.partner.runner.service;

import java.io.IOException;
import java.util.Date;
import java.util.Map;

import com.fasterxml.jackson.databind.ObjectMapper;
import org.jetbrains.annotations.NotNull;
import org.jooq.DSLContext;
import org.jooq.Record;
import org.jooq.Table;
import org.jooq.impl.DSL;
import org.json.JSONException;
import org.skyscreamer.jsonassert.JSONAssert;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Profile;
import org.springframework.stereotype.Component;

import ru.yandex.partner.libs.bs.json.PageBkDataComparator;

import static org.jooq.impl.DSL.field;
import static org.jooq.impl.DSL.max;
import static org.jooq.impl.DSL.select;

@Profile("page-bk-data-serialization")
@Component
public class PageBkDataSerializationCheckRunner extends BkDataCheckRunner<Void> {
    private static final Table<Record> BK_EDIT_PAGE_202X = DSL.table("partner_logs.bk_edit_page_2022");

    @Autowired
    private DSLContext dslContext;
    private PageBkDataComparator comparator = new PageBkDataComparator();
    private ObjectMapper om = new ObjectMapper();

    @NotNull
    @Override
    protected Map<Long, String> fetchBkData(long start) {
        var pageId = field("page_id", Long.class);
        var request = field("request", String.class);
        var dt = field("dt", Date.class);
        return dslContext.select(pageId, DSL.function("uncompress", String.class, request).as("request"))
                .from(BK_EDIT_PAGE_202X.as("bk"))
                .where(pageId.greaterThan(start)
                        .and(dt.eq(select(max(dt)).from(BK_EDIT_PAGE_202X.as("tmp_bk"))
                                .where(field("tmp_bk.page_id", Long.class).eq(field("bk.page_id", Long.class)))))
                )
                .orderBy(pageId)
                .limit(chunkSize)
                .fetchMap(pageId, request);
    }

    @Override
    protected void checkBkData(String bkData, Void container) throws IOException, JSONException {
        // bk data is weird here
        bkData = om.readTree(bkData).iterator().next().iterator().next().toString();
        var dataAfterConvert = bkDataConverter.convertPageBackAndForth(bkData);
        JSONAssert.assertEquals(bkData, dataAfterConvert, comparator);
    }
}
