package ru.yandex.partner.core.entity.block.actions.rtb.mobile.internal;

import java.util.Collection;
import java.util.List;

import com.fasterxml.jackson.databind.JsonNode;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import ru.yandex.partner.core.action.ActionPerformer;
import ru.yandex.partner.core.action.factories.CustomPayloadActionFactory;
import ru.yandex.partner.core.action.factories.SmartActionConfigurationFactory;
import ru.yandex.partner.core.action.factories.block.ActionStopFactory;
import ru.yandex.partner.core.entity.block.actions.BlockActionErrorHandler;
import ru.yandex.partner.core.entity.block.actions.BlockActionsEnum;
import ru.yandex.partner.core.entity.block.actions.all.BlockActionStop;
import ru.yandex.partner.core.entity.block.model.InternalMobileRtbBlock;
import ru.yandex.partner.core.entity.block.multistate.InternalMobileRtbBlockMultistateGraph;
import ru.yandex.partner.core.multistate.block.BlockStateFlag;

@Component
public class InternalMobileRtbBlockStopFactory
        extends CustomPayloadActionFactory<InternalMobileRtbBlock, BlockActionStop<InternalMobileRtbBlock>>
        implements ActionStopFactory<InternalMobileRtbBlock, BlockStateFlag> {
    private final InternalMobileRtbBlockMultistateGraph rtbBlockMultistateGraph;
    private final ActionPerformer actionPerformer;
    private final BlockActionErrorHandler<InternalMobileRtbBlock> errorHandler;

    @Autowired
    public InternalMobileRtbBlockStopFactory(InternalMobileRtbBlockMultistateGraph rtbBlockMultistateGraph,
                                             ActionPerformer actionPerformer,
                                             BlockActionErrorHandler<InternalMobileRtbBlock> errorHandler) {
        super(new SmartActionConfigurationFactory<>(
                BlockActionsEnum.STOP,
                rtbBlockMultistateGraph
        ));
        this.rtbBlockMultistateGraph = rtbBlockMultistateGraph;
        this.actionPerformer = actionPerformer;
        this.errorHandler = errorHandler;
    }

    @Override
    public BlockActionStop<InternalMobileRtbBlock> createAction(List<Long> ids, JsonNode payload) {
        return createAction(ids);
    }

    @Override
    public String getName() {
        return BlockActionsEnum.STOP.getActionName();
    }

    @Override
    public BlockActionStop<InternalMobileRtbBlock> createAction(Collection<Long> ids) {
        return new BlockActionStop<>(
                getActionConfiguration(),
                getName(),
                ids,
                rtbBlockMultistateGraph,
                actionPerformer,
                errorHandler,
                InternalMobileRtbBlock.class
        );
    }

    @Override
    public Class<InternalMobileRtbBlock> getModelClass() {
        return InternalMobileRtbBlock.class;
    }
}
